#!/bin/sh
# Copyright (c) 2000-2018 Synology Inc. All rights reserved.

PACKAGE_NAME="SafeAccess"
PACKAGE_TARGET_DIR="/var/packages/${PACKAGE_NAME}/target"
PACKAGE_WEBAPI_DIR="${PACKAGE_TARGET_DIR}/webapi"
PACKAGE_BIN_DIR="${PACKAGE_TARGET_DIR}/bin"
PACKAGE_CONF_DIR="/usr/syno/etc/packages/${PACKAGE_NAME}"
ACCESSCONTROL_DB_PATH="${PACKAGE_CONF_DIR}/synoaccesscontrol/database.db"
SYNOGETKEYVALUE="/usr/syno/bin/synogetkeyvalue"
SYNONETTOOL="/usr/syno/sbin/synonettool"
SYNOWEBAPI="/usr/syno/bin/synowebapi"
JQ="/bin/jq"
SQLITE3="/usr/bin/sqlite3"
LOG_FILE="/var/log/safeaccess.log"

info_log ()
{
	echo "[$(date +'%Y/%m/%d %H:%M:%S')][info] dnsfilter-util-change.sh: $1" >> ${LOG_FILE}
}

get_support_ipv6 ()
{
	local platform=`/bin/get_key_value /etc.defaults/synoinfo.conf unique | cut -d'_' -f2`
	if [ "xipq806x" = "x${platform}" -o "xdakota" = "x${platform}" -o "xcypress" = "x${platform}" ]; then
		echo "yes"
	else
		echo "no"
	fi
}

get_security_enabled()
{
	local is_security_enabled=`get_key_value ${PACKAGE_CONF_DIR}/synosafebrowsing/safe_browsing.conf enable_security`

	if [ "xyes" = "x${is_security_enabled}" ]; then
		echo "yes"
	else
		echo "no"
	fi
}

get_safe_browsing_enabled ()
{
	local is_safe_browsing_enabled=$(${SYNOGETKEYVALUE} ${PACKAGE_CONF_DIR}/synosafebrowsing/safe_browsing.conf enable_safe_browsing)

	if [ "xyes" = "x${is_safe_browsing_enabled}" ]; then
		echo "yes"
	else
		echo "no"
	fi
}

get_profile_enabled()
{
	if [ -f "${ACCESSCONTROL_DB_PATH}" ]; then
		local profile_num=$(${SQLITE3} -noheader ${ACCESSCONTROL_DB_PATH} "SELECT COUNT(*) FROM profile WHERE visible = 1;")

		if [ $profile_num -gt 0 ]; then
			echo "yes"
			exit 0
		fi
	fi

	echo "no"
}

get_profile_with_filter_configs_enabled ()
{
	if [ -f "${ACCESSCONTROL_DB_PATH}" ]; then
		# check user profile
		local enable_count_user=$(${SQLITE3} -noheader ${ACCESSCONTROL_DB_PATH} "SELECT COUNT(*) FROM profile WHERE (enable_safe_search = 1 OR enable_filter = 1) AND (name NOT LIKE '\$%\$');")

		# check network profile (lbr0, gbr0)
		local enable_count_network=$(${SQLITE3} -noheader ${ACCESSCONTROL_DB_PATH} "SELECT COUNT(*) FROM profile WHERE (enable_safe_search = 1 OR enable_filter = 1) AND (name = '\$lbr0\$' OR name = '\$gbr0\$');")

		if [ "$(($enable_count_user + $enable_count_network))" -gt 0 ]; then
			echo "yes"
			exit 0
		fi

		local topology="$(${SYNONETTOOL} --topology --get)"
		if [ "router" = "${topology}" ]; then
			# network profile (lbr2-4)
			local enable_count_others=$(${SQLITE3} -noheader ${ACCESSCONTROL_DB_PATH} "SELECT COUNT(*) FROM profile WHERE (enable_safe_search = 1 OR enable_filter = 1) AND (name LIKE '\$lbr%\$');")
		else
			# network profile (br2-4)
			local enable_count_others=$(${SQLITE3} -noheader ${ACCESSCONTROL_DB_PATH} "SELECT COUNT(*) FROM profile WHERE (enable_safe_search = 1 OR enable_filter = 1) AND (name LIKE '\$br%\$');")
		fi

		if [ "$enable_count_others" -gt 0 ]; then
			echo "yes"
			exit 0
		fi
	fi

	echo "no"
}

get_set_difference ()
{
	# Get $1 - $2 set
	# e.g. $1="lbr0 gbr0 lbr2 lbr3", $2="lbr0 gbr0 lbr3", output="lbr2"
	# e.g. $1="lbr0 gbr0",           $2="lbr0 gbr0 lbr3", output=""
	# Note that arguments should be quoted by "". E.g. get_set_difference "${var1}" "${var2}"
	echo "$1 $2 $2" | tr ' ' '\n' | sort | uniq -u | tr '\n' ' ' | sed 's/ *$//'
}

remove_network_profiles_of_interfaces ()
{
	# The format of argument should be like: "lbr2 lbr4"
	local ifnames="$1"
	if [ -z "${ifnames}" ]; then
		return
	fi

	local profile_get_result="$(${SYNOWEBAPI} --exec api=SYNO.SafeAccess.AccessControl.Profile method=get version=1 2>/dev/null)"
	local success=$(echo "${profile_get_result}" | ${JQ} -r '.success')
	if ! ${success} ; then
		return
	fi

	for ifname in ${ifnames} ; do
		local query=".data.profiles[] | select(.network == \"${ifname}\") | .profile_id"
		local profile_id=$(echo ${profile_get_result} | ${JQ} -r "${query}")
		if [ -z "${profile_id}" ]; then
			# This interface does not have network profile
			continue
		fi

		success=$(${SYNOWEBAPI} --exec api=SYNO.SafeAccess.AccessControl.Profile method=remove version=1 profile_id=${profile_id} 2>/dev/null | ${JQ} -r '.success')
		if ${success} ; then
			info_log "Removed profile: [network=${ifname}][profile_id=${profile_id}]"
		else
			info_log "Failed to remove profile: [network=${ifname}][profile_id=${profile_id}]"
		fi
	done
}

start_synosafeaccessfilterd_if_required ()
{
	if [ "xyes" = "x$(get_safe_browsing_enabled)" -o "xyes" = "x$(get_profile_with_filter_configs_enabled)" ]; then
		/sbin/start synosafeaccessfilterd
	fi
}

